#ifndef __globals_h__
#define __globals_h__

//Treibername



//registry-Eintrag fr Treiber
extern HKEY   KEY;
extern LPCSTR SUBKEY;

//Eventgruppe finden (siehe Events unten)
#define FindEventGroup(arg1) ( arg1 >> 8 )


//Plattform
typedef unsigned char* PBYTE;
typedef unsigned long  DWORD;



//Callback
typedef void (__stdcall *FN_CALLBACK)(WPARAM, LPARAM);



//****************************Message-Prioritt******************************************
enum Priority	{HIGH=1,		MEDIUM,			LOW};



//*********************************Events************************************************

enum EventGroups{AUDIO,			TUNER,			DIAL,			GENERAL,		
				 CANSJA,		POWER,			KLINE,			ERRORS};

//ATTENTION: Eventgroups above are associated to event-enums below

enum AudioEvents{AUDERROR = 0x000,	
				 VOLCAPS,		BALCAPS,		FADCAPS,		BASCAPS,		TREBCAPS,	
				 LOUDCAPS,		SRCCON,			VOLCON,			BALCON,			FADCON,	
				 BASCON,		TREBCON,		LOUDCON};


enum TunerEvents{TUNERROR = 0x100,	
				 TUNCAPS,		FREQCON,		BANDCON,		AREACON,		SCANRUN,
				 SCANCOMPL,		SCANERROR};


enum DialEvents {DIALERROR = 0x200,	
				 DIALPUSH,		DIALRELEASE,	DIALRIGHT,		DIALLEFT};


enum GeneralEvts{ERRORDLL = 0x300, 
				 DLLCONNECT,	DLLDISCONNECT, VERSION};


enum SjaEvents	{CANSJAERROR = 0x400,
				 PHYSCONNECT,	LOGCONNECT,		PHYSDISCONNECT,	LOGDISCONNECT,	DEACTIVATE,		
				 RXDATA,		TXDATA,			STATUSMSG,		CONNECTERROR};

	
enum PowerEvents{POWERERROR = 0x500,
				 SHUTDOWNEVT,	SHUTDOWNCON,	IGNITIONEVT,	IGNITIONCON,	BATCAPACITY};

				
enum KLineEvents{KLINEERROR = 0x600,
				 KPHYSCONNECT,	KLOGCONNECT,	KPHYSDISCON,	KLOGDISCON,		KDEACTIVATE,		
				 KRXDATA,		KTXDATA,		KSTATUSMSG,		KCAPSCON,		KCONNECTERROR,
				 KDISCONERROR,  KVERSION};


enum ErrorEvents{MSGERROR=0x700,
				 ERRGENERAL,	ERRRANGE};



//*****************************Eventparameter********************************************
enum Sources{	 SRCCD=1,		SRCSNDCARD,		SRCPHONE,		SRCMW,			SRCFM, 
				 MUTE};
//Band (entspricht Parameter von C167-Message)
enum Bands{		 FM=1,			SW,				MW,				LW,				WX};

//AreaCode (entspricht Parameter von C167-Message)
enum AreaCodes{  EUROPE=1,		USA,			EAST,			JAPAN};


enum Ignition{	 OFF,			ON};



//****************Alle Soundparameter, die sich abfragen lassen**************************
enum Item{		 VOLSTAT,		BALSTAT,		FADSTAT,		BASSTAT,		TREBSTAT, 
				 LOUDSTAT,		SRCSTAT,		FREQSTAT,		BANDSTAT,		AREASTAT};



//*******************DrehDrck direkt auf Hardware wirken lassen*************************
enum DialTarget{ TARGETNONE,	TARGETVOL,		TARGETBAL,		TARGETFAD,		TARGETFMEUR,
				 TARGETAMEUR,	TARGETFMUSA,	TARGETAMUSA};



//***************************************************************************************
typedef struct tag_FirmwareVersion{
	DWORD	dwVersionDevice;
	DWORD	dwVersionFramework;
	DWORD	dwVersionCompatibility;
	DWORD	dwVersionModule;
} FWVERSION, *FWVERSION_P;



//**************************Bereichseinstellungen Audio**********************************
typedef struct AudioCaps{
	int volMin;
	int volMax;

	int balLeft;
	int balRight;
	int balCntr;

	int fadFront;
	int fadRear;
	int fadCntr;

	int basMin;
	int basMax;
	int basCntr;
	unsigned int basCntrFreq;

	int trebMin;
	int trebMax;
	int trebCntr;
	unsigned int trebCntrFreq;

	int loudMin;
	int loudMax;
	int loudCntr;
	unsigned int loudCntrFreq;
} AUDIOCAPS;



//********************************Frequenzparameter**************************************
typedef struct FreqParams{
	unsigned int frequency;
	unsigned int rds;
	unsigned int stereo;	
} FREQPARAMS;

//Bereichseinstellungen Tuner
typedef struct BandDescription{
	unsigned int bandType;
	unsigned int minFreq;
	unsigned int maxFreq;
	unsigned int deltaF;
	
} BANDDESCR;

//Bereichseinstellungen Tuner
typedef struct AreaDescription{
	unsigned int areaCode; //one of 1, 2, 3, 4
	BANDDESCR	 band[4];
} AREADESCR;

//Bandbeschreibung (Feldstrke, RDS, Stereo)
typedef struct FrequencyInfo{
	unsigned int frequency;
	unsigned int signalQuality;
	unsigned int stereo;
	unsigned int rds;
} FREQUINFO;

#define FLIST_ITEMS		(2100)

typedef struct FrequencyList{
	FREQUINFO	item[FLIST_ITEMS];
	unsigned int nEntries;
} FREQULIST;


//******************************CAN-Verbindungsinformationen*****************************
typedef struct CANConnectionInfo{
	unsigned int physicalId;
	unsigned int logicalId;
	unsigned int speedInKBit;
	unsigned int canMode;		//PELICAN or BASICCAN
	DWORD		 filter;		//MessageId to filter
	DWORD		 filterMask;	//Mask on MessageId filter
	bool		 filterMode;	//ignore or accept messages that pass the filter
	unsigned int connectState;	//status
	BYTE		 connectionFlags;
} CANINFO;

typedef struct CANStatus{
	unsigned int	CANbusState;
	unsigned int	rxErrorCount;
	unsigned int	txErrorCount;
	unsigned int	errorDescriptor;
} CANSTATUS;

#define		BASICCAN		(1)
#define		PELICAN			(2)
#define		MSGIGNORE		(false)
#define		MSGACCEPT		(true)

enum CANStates{DISCONNECTED, PHYSICAL_RDY, LOGICAL_RDY};

typedef struct tag_CanData{
	DWORD		canId;
	DWORD		rxFlags;
	unsigned char	msg[8];
	unsigned int	msgLen;
} CANDATA;



//******************************K-Line Verbindungsinformationen**************************

typedef struct tag_KLineConnectionInfo{
	DWORD		physicalId;
	DWORD		logicalId;
	DWORD		protocolId;
	DWORD		connectState;
	DWORD		disconnectReason;

	DWORD		busSpeeds;
	DWORD		txFlags;
	DWORD		logConnectionType;
	DWORD		physConnectionType;
	DWORD		logConnectionParams;

	DWORD		dwVersionMayor;
	DWORD		dwVersionMinor;
} KLINEINFO, *KLINEINFO_P;

typedef struct tag_KLineData{
	unsigned char	message[512];
	unsigned char	title;
	unsigned char*	pData;
	unsigned int	msgLen;
} KLINEDATA, *KLINEDATA_P;

typedef struct tag_KLineAnswer{
	DWORD   		dwValue;
	unsigned int	uiChannel;
} KLINEANSWER, *KLINEANSWER_P;


#endif  //__globals_h__



