/****************************************************************************************
 *
 * FILE         $Workfile: carmsg.h $
 *				$Archive: /Components/CarAPI/WIN32DLL/Source/carmsg.h $
 * 
 * Version:     $Revision: 10 $ 
 * Date:        $Date: 17.09.02 17:40 $
 * LastAuthor:  $Author: Bjstichn $
 *
 * Author:      Jrg Stichnoth
 *
 * --------------------------------------------------------------------------------------
 * !MODULE      [CAR-API INTERFACE]
 * --------------------------------------------------------------------------------------
 *
 *         Copyright (c)         2002 iav gmbh
 *                               carnotstrasse 1
 *                               10587 berlin
 *                               Phone: +49 30 39978-0
 *                               http://www.iav.de
 *         All Rights Reserved
 *
 * --------------------------------------------------------------------------------------
 * !DESCRIPTION
 *
 *
 * --------------------------------------------------------------------------------------
 * !HISTORY
 *  is located at the end of this file 
 *
 ***************************************************************************************/

#ifndef _CARMSG_H_
#define _CARMSG_H_

// ============================================================================
//
// predefines
//
// ============================================================================

// error codes
#define CARAPI_NO_ERROR							(0)   // success
#define CARAPI_ERROR_FILE_NOT_FOUND             (2)   // the device was not found
#define CARAPI_ERROR_ACCESS_DENIED              (5)   // access to the specified object was denied
#define CARAPI_ERROR_INVALID_HANDLE             (6)   // object handle is not valid
#define CARAPI_ERROR_INVALID_DATA               (13)  // parameter missing or parameter contents invalid
#define CARAPI_ERROR_OUTOFMEMORY                (14)  // not enough memory to complete command
#define CARAPI_ERROR_NOT_READY                  (21)  // device is not ready
#define CARAPI_ERROR_BAD_LENGTH                 (24)  // command block is inconsistent
#define CARAPI_ERROR_WRITE_FAULT                (29)  // cannot write to device
#define CARAPI_ERROR_READ_FAULT                 (30)  // cannot read from device
#define CARAPI_ERROR_NOT_SUPPORTED              (50)  // command is not supported
#define CARAPI_ERROR_BUSY                       (170) // device is busy
#define CARAPI_ERROR_UNSPECIFIED				CARAPI_ERROR_NOT_READY // unspecified error

// macros
#define CARAPI_CREATE_INTERFACE_CLASS(UnitClass, UnitSubclass, UnitType) (DWORD)((((DWORD)UnitClass & 0xff) << 24) | (((DWORD)UnitSubclass & 0xff) << 16) | (((DWORD)UnitType & 0xff) << 8))
#define CARAPI_CREATE_MESSAGE(InterfaceClass, MessageID, MessageType) (DWORD)(((DWORD)InterfaceClass & 0xffffff00) | (((DWORD)MessageID & 0x3f) << 2) | ((DWORD)MessageType & 0x03))
#define CARAPI_MSG_GETUNITCLASS(Msg) ((Msg >> 24) & 0xff)
#define CARAPI_MSG_GETUNITSUBCLASS(Msg) ((Msg >> 16) & 0xff)
#define CARAPI_MSG_GETUNITTYPE(Msg) ((Msg >> 8) & 0xff)
#define CARAPI_MSG_GETMSGCMD(Msg) ((Msg >> 2) & 0x3f)
#define CARAPI_MSG_GETMSGTYPE(Msg) (Msg & 0x03)

#define CARAPI_MSG_SETMSGTYPE(Msg,type) (Msg = (Msg & 0xFFFFFFFC) | type)

// base classes
#define UnitClass_Base                          (0)
#define UnitClass_uC                            (1)
#define UnitClass_Power                         (2)
#define UnitClass_Hid                           (3)
#define UnitClass_Audio                         (4)
#define UnitClass_IOBus	                        (5)

// subclasses
#define UnitSubclass_Base                       (0)

#define UnitSubclass_uC_CPU                     (1)
#define UnitSubclass_uC_Flash                   (2)

#define UnitSubclass_Power_Ignition             (1)
#define UnitSubclass_Power_Battery              (2)
#define UnitSubclass_Power_Supply               (3)

#define UnitSubclass_Hid_Key                    (1)
#define UnitSubclass_Hid_Dial                   (2)
#define UnitSubclass_Hid_Switch                 (3)

#define UnitSubclass_Audio_Speaker              (1)
#define UnitSubclass_Audio_Mixer                (2)
#define UnitSubclass_Audio_Attenuator           (3)
#define UnitSubclass_Audio_Filter               (4)
#define UnitSubclass_Audio_Tuner                (5)
#define UnitSubclass_Audio_CDPlayer             (6)

#define UnitSubclass_IOBus_CAN		            (1)
#define UnitSubclass_IOBus_MOST		            (2)
#define UnitSubclass_IOBus_USB		            (3)

// types
#define UnitType_Standard                       (0)

#define UnitType_AudioMixer_Selector            (1)
#define UnitType_AudioMixer_Mix                 (2)

#define UnitType_AudioAttenuator_Volume         (1)
#define UnitType_AudioAttenuator_Balance        (2)
#define UnitType_AudioAttenuator_Fader          (3)

#define UnitType_AudioFilter_Mute               (1)
#define UnitType_AudioFilter_Dolby              (2)
#define UnitType_AudioFilter_Loudness           (3)
#define UnitType_AudioFilter_MPX                (4)
#define UnitType_AudioFilter_Bass               (5)
#define UnitType_AudioFilter_Treble             (6)
#define UnitType_AudioFilter_Parametric         (7)

// defines
#define UnitClass_BaseDefault                   CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Base, UnitSubclass_Base, UnitType_Standard)

#define UnitClass_uC_CPU                        CARAPI_CREATE_INTERFACE_CLASS(UnitClass_uC, UnitSubclass_uC_CPU, UnitType_Standard)
#define UnitClass_uC_Flash                      CARAPI_CREATE_INTERFACE_CLASS(UnitClass_uC, UnitSubclass_uC_Flash, UnitType_Standard)

#define UnitClass_PowerIgnition                 CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Power, UnitSubclass_Power_Ignition, UnitType_Standard)
#define UnitClass_PowerBattery                  CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Power, UnitSubclass_Power_Battery, UnitType_Standard)
#define UnitClass_PowerSupply                   CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Power, UnitSubclass_Power_Supply, UnitType_Standard)

#define UnitClass_Hid_Key                       CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Hid, UnitSubclass_Hid_Key, UnitType_Standard)
#define UnitClass_Hid_Dial                      CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Hid, UnitSubclass_Hid_Dial, UnitType_Standard)
#define UnitClass_Hid_Switch                    CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Hid, UnitSubclass_Hid_Switch, UnitType_Standard)

#define UnitClass_Audio_Speaker                 CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Speaker, UnitType_Standard)
#define UnitClass_Audio_Selector                CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Mixer, UnitType_AudioMixer_Selector)
#define UnitClass_Audio_Mixer                   CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Mixer, UnitType_AudioMixer_Mixer)
#define UnitClass_Audio_Volume                  CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Attenuator, UnitType_AudioAttenuator_Volume)
#define UnitClass_Audio_Balance                 CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Attenuator, UnitType_AudioAttenuator_Balance)
#define UnitClass_Audio_Fader                   CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Attenuator, UnitType_AudioAttenuator_Fader)
#define UnitClass_Audio_FilterLoudness          CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Filter, UnitType_AudioFilter_Loudness)
#define UnitClass_Audio_FilterBass              CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Filter, UnitType_AudioFilter_Bass)
#define UnitClass_Audio_FilterTreble            CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Filter, UnitType_AudioFilter_Treble)
#define UnitClass_Audio_FilterParametric        CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Filter, UnitType_AudioFilter_Parametric)
#define UnitClass_Audio_Tuner                   CARAPI_CREATE_INTERFACE_CLASS(UnitClass_Audio, UnitSubclass_Audio_Tuner, UnitType_Standard)

#define UnitClass_IO_CAN                        CARAPI_CREATE_INTERFACE_CLASS(UnitClass_IOBus, UnitSubclass_IOBus_CAN, UnitType_Standard)

#define UnitID_Broadcast                        (-1)


#define MsgType_CMD                             (1)
#define MsgType_CON                             (2)
#define MsgType_EVT                             (3)


#define InterfaceMsg_EnumDevices                (2)
#define InterfaceMsg_EnumConnections            (3)
#define InterfaceMsg_RegisterClient             (4)
#define InterfaceMsg_UnregisterClient           (5)

#define InterfaceMsg_Flash_Read                 (2)
#define InterfaceMsg_Flash_Write                (3)

#define InterfaceMsg_PowerIgnition_State        (2)
#define InterfaceMsg_PowerBattery_Level         (3)
#define InterfaceMsg_PowerSupply_SwitchOff      (4)

#define InterfaceMsg_HidKey_Up                  (2)
#define InterfaceMsg_HidKey_Down                (3)
#define InterfaceMsg_HidDial_Left               (2)
#define InterfaceMsg_HidDial_Right              (3)

#define InterfaceMsg_AudioSelector_QueryCaps    (1)
#define InterfaceMsg_AudioSelector_Position     (2)

#define InterfaceMsg_AudioAttenuator_QueryCaps  (1)
#define InterfaceMsg_AudioAttenuator_Position   (2)

#define InterfaceMsg_AudioFilterBass_QueryCaps  (1)
#define InterfaceMsg_AudioFilterBass_Position   (2)

#define InterfaceMsg_AudioFilterTreble_QueryCaps (1)
#define InterfaceMsg_AudioFilterTreble_Position  (2)

#define InterfaceMsg_AudioFilterLoudness_QueryCaps (1)
#define InterfaceMsg_AudioFilterLoudness_Position  (2)

#define InterfaceMsg_AudioFilterParametric_QueryCaps     (1)
#define InterfaceMsg_AudioFilterParametric_Position      (2)
#define InterfaceMsg_AudioFilterParametric_Frequency     (3)
#define InterfaceMsg_AudioFilterParametric_QualityFactor (4)

#define InterfaceMsg_AudioTuner_QueryCaps       (1)
#define InterfaceMsg_AudioTuner_SelectArea      (2)
#define InterfaceMsg_AudioTuner_SelectBand      (3)
#define InterfaceMsg_AudioTuner_SetFrequency    (4)
#define InterfaceMsg_AudioTuner_ScanBand        (5)

#define InterfaceMsg_IOCAN_QueryCaps            (1)
#define InterfaceMsg_IOCAN_PhysicalConnect      (2)
#define InterfaceMsg_IOCAN_PhysicalDisconnect   (3)
#define InterfaceMsg_IOCAN_LogicalConnect       (4)
#define InterfaceMsg_IOCAN_LogicalParams        (5)
#define InterfaceMsg_IOCAN_LogicalStatus        (6)
#define InterfaceMsg_IOCAN_LogicalDisconnect    (7)
#define InterfaceMsg_IOCAN_Send                 (8)
#define InterfaceMsg_IOCAN_Recv                 (9)

// ============================================================================
//
// Messages
//
// ============================================================================

// General Group
// Identify
#define CARMSG_EnumDevices_CMD                         CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_EnumDevices, MsgType_CMD)
#define CARMSG_EnumDevices_EVT                         CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_EnumDevices, MsgType_EVT)
#define CARMSG_EnumDevices_CON                         CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_EnumDevices, MsgType_CON)
// Client Registration
#define CARMSG_RegisterClient_CMD                      CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_RegisterClient, MsgType_CMD)
#define CARMSG_RegisterClient_CON                      CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_RegisterClient, MsgType_CMD)
#define CARMSG_UnregisterClient_CMD                    CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_UnregisterClient, MsgType_CMD)
#define CARMSG_UnregisterClient_CON                    CARAPI_CREATE_MESSAGE(UnitClass_Base, InterfaceMsg_UnregisterClient, MsgType_CMD)

// CPU
// Flash
#define CARMSG_Flash_Read_CMD                          CARAPI_CREATE_MESSAGE(UnitClass_uC_Flash, InterfaceMsg_Flash_Read, MsgType_CMD)
#define CARMSG_Flash_Read_CON                          CARAPI_CREATE_MESSAGE(UnitClass_uC_Flash, InterfaceMsg_Flash_Read, MsgType_CON)
#define CARMSG_Flash_Write_CMD                         CARAPI_CREATE_MESSAGE(UnitClass_uC_Flash, InterfaceMsg_Flash_Write, MsgType_CMD)
#define CARMSG_Flash_Write_CON                         CARAPI_CREATE_MESSAGE(UnitClass_uC_Flash, InterfaceMsg_Flash_Write, MsgType_CON)

// Power
// Ignition
#define CARMSG_Ignition_State_CMD                      CARAPI_CREATE_MESSAGE(UnitClass_PowerIgnition, InterfaceMsg_PowerIgnition_State, MsgType_CMD)
#define CARMSG_Ignition_State_EVT                      CARAPI_CREATE_MESSAGE(UnitClass_PowerIgnition, InterfaceMsg_PowerIgnition_State, MsgType_EVT)
#define CARMSG_Ignition_State_CON                      CARAPI_CREATE_MESSAGE(UnitClass_PowerIgnition, InterfaceMsg_PowerIgnition_State, MsgType_CON)
// Battery
#define CARMSG_Battery_Level_CMD                       CARAPI_CREATE_MESSAGE(UnitClass_PowerBattery, InterfaceMsg_PowerBattery_Level, MsgType_CMD)
#define CARMSG_Battery_Level_EVT                       CARAPI_CREATE_MESSAGE(UnitClass_PowerBattery, InterfaceMsg_PowerBattery_Level, MsgType_EVT)
#define CARMSG_Battery_Level_CON                       CARAPI_CREATE_MESSAGE(UnitClass_PowerBattery, InterfaceMsg_PowerBattery_Level, MsgType_CON)
// Power Supply
#define CARMSG_PowerSupply_SwitchOff_CMD               CARAPI_CREATE_MESSAGE(UnitClass_PowerSupply, InterfaceMsg_PowerSupply_SwitchOff, MsgType_CMD)
#define CARMSG_PowerSupply_SwitchOff_EVT               CARAPI_CREATE_MESSAGE(UnitClass_PowerSupply, InterfaceMsg_PowerSupply_SwitchOff, MsgType_EVT)
#define CARMSG_PowerSupply_SwitchOff_CON               CARAPI_CREATE_MESSAGE(UnitClass_PowerSupply, InterfaceMsg_PowerSupply_SwitchOff, MsgType_CON)

// HID Group
// Key
#define CARMSG_Key_Up_EVT                              CARAPI_CREATE_MESSAGE(UnitClass_Hid_Key, InterfaceMsg_HidKey_Up, MsgType_EVT)
#define CARMSG_Key_Down_EVT                            CARAPI_CREATE_MESSAGE(UnitClass_Hid_Key, InterfaceMsg_HidKey_Down, MsgType_EVT)
// Dial
#define CARMSG_Dial_TurnLeft_EVT                       CARAPI_CREATE_MESSAGE(UnitClass_Hid_Dial, InterfaceMsg_HidDial_Left, MsgType_EVT)
#define CARMSG_Dial_TurnRight_EVT                      CARAPI_CREATE_MESSAGE(UnitClass_Hid_Dial, InterfaceMsg_HidDial_Right, MsgType_EVT)

// Audio Group
// Selector
#define CARMSG_AudioSelector_QueryCaps_CMD             CARAPI_CREATE_MESSAGE(UnitClass_Audio_Selector, InterfaceMsg_AudioSelector_QueryCaps, MsgType_CMD)
#define CARMSG_AudioSelector_QueryCaps_CON             CARAPI_CREATE_MESSAGE(UnitClass_Audio_Selector, InterfaceMsg_AudioSelector_QueryCaps, MsgType_CON)
#define CARMSG_AudioSelector_Position_CMD              CARAPI_CREATE_MESSAGE(UnitClass_Audio_Selector, InterfaceMsg_AudioSelector_Position, MsgType_CMD)
#define CARMSG_AudioSelector_Position_CON              CARAPI_CREATE_MESSAGE(UnitClass_Audio_Selector, InterfaceMsg_AudioSelector_Position, MsgType_CON)
// Volume
#define CARMSG_AudioVolume_QueryCaps_CMD               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Volume, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CMD)
#define CARMSG_AudioVolume_QueryCaps_CON               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Volume, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CON)
#define CARMSG_AudioVolume_Position_CMD                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Volume, InterfaceMsg_AudioAttenuator_Position, MsgType_CMD)
#define CARMSG_AudioVolume_Position_CON                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Volume, InterfaceMsg_AudioAttenuator_Position, MsgType_CON)
// Balance
#define CARMSG_AudioBalance_QueryCaps_CMD              CARAPI_CREATE_MESSAGE(UnitClass_Audio_Balance, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CMD)
#define CARMSG_AudioBalance_QueryCaps_CON              CARAPI_CREATE_MESSAGE(UnitClass_Audio_Balance, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CON)
#define CARMSG_AudioBalance_Position_CMD               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Balance, InterfaceMsg_AudioAttenuator_Position, MsgType_CMD)
#define CARMSG_AudioBalance_Position_CON               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Balance, InterfaceMsg_AudioAttenuator_Position, MsgType_CON)
// Fader
#define CARMSG_AudioFader_QueryCaps_CMD                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Fader, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CMD)
#define CARMSG_AudioFader_QueryCaps_CON                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Fader, InterfaceMsg_AudioAttenuator_QueryCaps, MsgType_CON)
#define CARMSG_AudioFader_Position_CMD                 CARAPI_CREATE_MESSAGE(UnitClass_Audio_Fader, InterfaceMsg_AudioAttenuator_Position, MsgType_CMD)
#define CARMSG_AudioFader_Position_CON                 CARAPI_CREATE_MESSAGE(UnitClass_Audio_Fader, InterfaceMsg_AudioAttenuator_Position, MsgType_CON)
// Bass Filter
#define CARMSG_AudioFilterBass_QueryCaps_CMD           CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterBass, InterfaceMsg_AudioFilterBass_QueryCaps, MsgType_CMD)
#define CARMSG_AudioFilterBass_QueryCaps_CON           CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterBass, InterfaceMsg_AudioFilterBass_QueryCaps, MsgType_CON)
#define CARMSG_AudioFilterBass_Position_CMD            CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterBass, InterfaceMsg_AudioFilterBass_Position, MsgType_CMD)
#define CARMSG_AudioFilterBass_Position_CON            CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterBass, InterfaceMsg_AudioFilterBass_Position, MsgType_CON)
// Treble Filter
#define CARMSG_AudioFilterTreble_QueryCaps_CMD         CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterTreble, InterfaceMsg_AudioFilterTreble_QueryCaps, MsgType_CMD)
#define CARMSG_AudioFilterTreble_QueryCaps_CON         CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterTreble, InterfaceMsg_AudioFilterTreble_QueryCaps, MsgType_CON)
#define CARMSG_AudioFilterTreble_Position_CMD          CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterTreble, InterfaceMsg_AudioFilterTreble_Position, MsgType_CMD)
#define CARMSG_AudioFilterTreble_Position_CON          CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterTreble, InterfaceMsg_AudioFilterTreble_Position, MsgType_CON)
// Loudness Filter
#define CARMSG_AudioFilterLoudness_QueryCaps_CMD       CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterLoudness, InterfaceMsg_AudioFilterLoudness_QueryCaps, MsgType_CMD)
#define CARMSG_AudioFilterLoudness_QueryCaps_CON       CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterLoudness, InterfaceMsg_AudioFilterLoudness_QueryCaps, MsgType_CON)
#define CARMSG_AudioFilterLoudness_Position_CMD        CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterLoudness, InterfaceMsg_AudioFilterLoudness_Position, MsgType_CMD)
#define CARMSG_AudioFilterLoudness_Position_CON        CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterLoudness, InterfaceMsg_AudioFilterLoudness_Position, MsgType_CON)
// Parametric Filter
#define CARMSG_AudioFilterParametric_QueryCaps_CMD     CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_QueryCaps, MsgType_CMD)
#define CARMSG_AudioFilterParametric_QueryCaps_CON     CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_QueryCaps, MsgType_CON)
#define CARMSG_AudioFilterParametric_Position_CMD      CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_Position, MsgType_CMD)
#define CARMSG_AudioFilterParametric_Position_CON      CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_Position, MsgType_CON)
#define CARMSG_AudioFilterParametric_Frequency_CMD     CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_Frequency, MsgType_CMD)
#define CARMSG_AudioFilterParametric_Frequency_CON     CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_Frequency, MsgType_CON)
#define CARMSG_AudioFilterParametric_QualityFactor_CMD CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_QualityFactor, MsgType_CMD)
#define CARMSG_AudioFilterParametric_QualityFactor_CON CARAPI_CREATE_MESSAGE(UnitClass_Audio_FilterParametric, InterfaceMsg_AudioFilterParametric_QualityFactor, MsgType_CON)
// Tuner
#define CARMSG_AudioTuner_QueryCaps_CMD                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_QueryCaps, MsgType_CMD)
#define CARMSG_AudioTuner_QueryCaps_CON                CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_QueryCaps, MsgType_CON)
#define CARMSG_AudioTuner_SelectArea_CMD               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SelectArea, MsgType_CMD)
#define CARMSG_AudioTuner_SelectArea_CON               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SelectArea, MsgType_CON)
#define CARMSG_AudioTuner_SelectBand_CMD               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SelectBand, MsgType_CMD)
#define CARMSG_AudioTuner_SelectBand_CON               CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SelectBand, MsgType_CON)
#define CARMSG_AudioTuner_SetFrequency_CMD             CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SetFrequency, MsgType_CMD)
#define CARMSG_AudioTuner_SetFrequency_CON             CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_SetFrequency, MsgType_CON)
#define CARMSG_AudioTuner_ScanBand_CMD                 CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_ScanBand, MsgType_CMD)
#define CARMSG_AudioTuner_ScanBand_EVT                 CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_ScanBand, MsgType_EVT)
#define CARMSG_AudioTuner_ScanBand_CON                 CARAPI_CREATE_MESSAGE(UnitClass_Audio_Tuner, InterfaceMsg_AudioTuner_ScanBand, MsgType_CON)

// IOBus Group
// CAN
#define CARMSG_IOCAN_QueryCaps_CMD                     CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_QueryCaps, MsgType_CMD)
#define CARMSG_IOCAN_QueryCaps_CON                     CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_QueryCaps, MsgType_CON)
#define CARMSG_IOCAN_PhysicalConnect_CMD               CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalConnect, MsgType_CMD)
#define CARMSG_IOCAN_PhysicalConnect_CON               CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalConnect, MsgType_CON)
#define CARMSG_IOCAN_PhysicalConnect_EVT               CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalConnect, MsgType_EVT)
#define CARMSG_IOCAN_PhysicalDisconnect_CMD            CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalDisconnect, MsgType_CMD)
#define CARMSG_IOCAN_PhysicalDisconnect_CON            CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalDisconnect, MsgType_CON)
#define CARMSG_IOCAN_PhysicalDisconnect_EVT            CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_PhysicalDisconnect, MsgType_EVT)
#define CARMSG_IOCAN_LogicalConnect_CMD                CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalConnect, MsgType_CMD)
#define CARMSG_IOCAN_LogicalConnect_CON                CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalConnect, MsgType_CON)
#define CARMSG_IOCAN_LogicalConnect_EVT                CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalConnect, MsgType_EVT)
#define CARMSG_IOCAN_LogicalParams_CMD                 CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalParams, MsgType_CMD)
#define CARMSG_IOCAN_LogicalParams_CON                 CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalParams, MsgType_CON)
#define CARMSG_IOCAN_LogicalStatus_CMD                 CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalStatus, MsgType_CMD)
#define CARMSG_IOCAN_LogicalStatus_CON                 CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalStatus, MsgType_CON)
#define CARMSG_IOCAN_LogicalStatus_EVT                 CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalStatus, MsgType_EVT)
#define CARMSG_IOCAN_LogicalDisconnect_CMD             CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalDisconnect, MsgType_CMD)
#define CARMSG_IOCAN_LogicalDisconnect_CON             CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalDisconnect, MsgType_CON)
#define CARMSG_IOCAN_LogicalDisconnect_EVT             CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_LogicalDisconnect, MsgType_EVT)
#define CARMSG_IOCAN_Send_CMD                          CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_Send, MsgType_CMD)
#define CARMSG_IOCAN_Send_CON                          CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_Send, MsgType_CON)
#define CARMSG_IOCAN_Recv_EVT                          CARAPI_CREATE_MESSAGE(UnitClass_IO_CAN, InterfaceMsg_IOCAN_Recv, MsgType_EVT)

// block coding defines
#define CARAPI_CONTAINERTYPE_MASK       0x3f
#define CARAPI_CONTAINERTYPE_UINT       0x00
#define CARAPI_CONTAINERTYPE_SINT       0x01
#define CARAPI_CONTAINERTYPE_UINT8      0x02
#define CARAPI_CONTAINERTYPE_SINT8      0x03
#define CARAPI_CONTAINERTYPE_UINT16     0x04
#define CARAPI_CONTAINERTYPE_SINT16     0x05
#define CARAPI_CONTAINERTYPE_UINT32     0x06
#define CARAPI_CONTAINERTYPE_SINT32     0x07
#define CARAPI_CONTAINERTYPE_UINT64     0x08
#define CARAPI_CONTAINERTYPE_SINT64     0x09
#define CARAPI_CONTAINERTYPE_UINT128    0x0a
#define CARAPI_CONTAINERTYPE_SINT128    0x0b
#define CARAPI_CONTAINERTYPE_FLOAT      0x10
#define CARAPI_CONTAINERTYPE_DOUBLE     0x11
#define CARAPI_CONTAINERTYPE_POINTER    0x12
#define CARAPI_CONTAINERTYPE_FARPOINTER 0x13
#define CARAPI_CONTAINERTYPE_BINARYDATA 0x21
#define CARAPI_CONTAINERTYPE_CONTAINER  0x22

// ============================================================================
//
// CARAPI message functions
//
// ============================================================================
#ifndef CARAPI
	#define CARAPI __stdcall
#endif // CARAPI

#define BOOLEAN_OK TRUE
#define BOOLEAN_TRUE TRUE
#define BOOLEAN_FAIL FALSE
#define BOOLEAN_FALSE FALSE

BOOLEAN CARAPI carapiBlockEncValue (
	PBYTE		pBlockData,
	DWORD		dwBufferSize,
	PVOID		pValue,
	DWORD		dwValueBufferSize,
	BOOLEAN		bSigned,
	DWORD		*pdwElementSize);
BOOLEAN CARAPI carapiBlockDecValue (
	PBYTE		pBlockData, 
	PVOID		pValue,
	DWORD		nValueBufferSize,
	BOOLEAN		*pbSigned,
	DWORD		*pdwConsumedBytes);
BOOLEAN CARAPI carapiBlockEncBlock (
	PBYTE		pBlockData,
	DWORD		dwBufferSize,
	PBYTE		pBlockContents,
	DWORD		dwContentsLength,
	DWORD		dwBlockType,
	DWORD		*pdwLength);
BOOLEAN CARAPI carapiBlockDecBlock (
	PBYTE		pBlockData,
	PBYTE		*ppBlockContents,
	DWORD		*pdwContentsLength,
	DWORD		*pdwBlockType);
BOOLEAN carapiBlockEncf (
	PBYTE		pBlockData,
	DWORD		dwBufferSize,
	PCSTR		szFormat,
	...);
BOOLEAN carapiBlockDecf (
	PBYTE		pBlockData,
	PCSTR		szFormat,
	...);
BOOLEAN CARAPI carapiBlockGetLength (
	PBYTE		pBlockData,
	DWORD		*pdwLength);
BOOLEAN CARAPI carapiBlockEncBlockHeaderRev (
	PBYTE		pBlockData,
	DWORD		dwBufferSize,
	DWORD		dwContentsLength,
	DWORD		dwBlockType,
	DWORD		*pdwLength);

#endif  // _CARMSG_H_

 /***************************************************************************************
 *
 * $History: carmsg.h $  
 * 
 * *****************  Version 10  *****************
 * User: Bjstichn     Date: 17.09.02   Time: 17:40
 * Updated in $/Components/CarAPI/WIN32DLL/Source
 * added register client message (needed for power management)
 * 
 * *****************  Version 9  *****************
 * User: Bfengel      Date: 12.09.02   Time: 14:03
 * Updated in $/Projects/IAV GmbH/TegTester
 * 
 * *****************  Version 8  *****************
 * User: Bjstichn     Date: 1.08.02    Time: 15:24
 * Updated in $/Projects/Claas/Cebis I/Application/cebis/carapi/WIN32DLL/Source
 * changed message format
 * 
 * *****************  Version 7  *****************
 * User: Bjstichn     Date: 23.07.02   Time: 19:12
 * Updated in $/Projects/Claas/Cebis I/Application/cebis/carapi/WIN32DLL/Source
 * 
 * *****************  Version 6  *****************
 * User: Bjstichn     Date: 6.05.02    Time: 11:39
 * Updated in $/Components/CarAPI/Driver/Version 1.0/Source/shared/carapi
 * 
 * *****************  Version 5  *****************
 * User: Btpaul       Date: 2.05.02    Time: 18:30
 * Updated in $/Components/CarAPI/Driver/Version 1.0/Source/shared/CarApi
 * 
 * *****************  Version 4  *****************
 * User: Btpaul       Date: 2.05.02    Time: 10:24
 * Updated in $/Components/CarAPI/Driver/Version 1.0/Source/shared/CarApi
 * 
 * *****************  Version 3  *****************
 * User: Btpaul       Date: 26.04.02   Time: 18:07
 * Updated in $/Components/CarAPI/Driver/Version 1.0/Source/shared/CarApi
 * 
 * *****************  Version 2  *****************
 * User: Btpaul       Date: 26.04.02   Time: 17:34
 * Updated in $/Components/CarAPI/Driver/Version 1.0/Source/shared/CarApi
 * Driver Framework support added
 * 
 ***************************************************************************************/

