#include "stdafx.h"
#include "windows.h"
#include <stdio.h>

#include "pm.h"
#include "pmService.h"

#ifdef __cplusplus
extern "C" 
{
	#include "carapi.h"
}
#endif	// __cplusplus

DWORD	m_linkId;
bool	m_connected;
int		m_delay;

#define SWTICHOFF_DEFAULT	(50000)

/*****************************************************************************/
/*
/*  Funktion:		main
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		The service can listen to one of three parameters at a time:
/*					'install' or 'uninstall' or x where x is an integer that holds
/*					the switchoff delay in ms.
/*					If no parameter is shipped the service is started with the
/*					default switchoff-delay of 50000 ms
/*					the command line call would be: 'pm.exe install'
/*													'pm.exe uninstall' 			
/*		
/*****************************************************************************/
int main(int argc, char* argv[])
{	
	m_delay = SWTICHOFF_DEFAULT;

	if (argc > 2){

		DBGTRACE(("Main - too much arguments"));
		printf("Too much arguments.\n");
        printf("Available Parameters:\n");
        printf("  install: installs the service.\n");
        printf("  uninstall: uninstalls the service.\n");
    }
    else if (argc == 2){

        if (lstrcmpi(argv[1], TEXT("install")) == 0){	
			DBGTRACE(("Main - InstallService called"));
            InstallService();
			exit(0);
        }
        else if (lstrcmpi(argv[1], TEXT("uninstall")) == 0){	
			DBGTRACE(("Main - UninstallService called"));
            UninstallService();
			exit(0);
        }
		else{

			m_delay = atoi(argv[1]);

			if ( (m_delay > 5000) && (m_delay < 180000) ){
				DBGTRACE(("Main - poweroff delay is %d ms", m_delay));
				RunService();
			}
			else{
				DBGTRACE(("Main - unrecognized argument"));	
				printf("Too much arguments.\n");
				printf("Available Parameters:\n");
				printf("  install: installs the service.\n");
				printf("  uninstall: uninstalls the service.\n");
				exit(0);
			}
		}
    }
	else{
		DBGTRACE(("Main - poweroff delay set to default: %d ms", m_delay));
        RunService();
    }

	return 0;
}


/*****************************************************************************/
/*
/*  Funktion:		Connect
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		We need to connect to the carapi.dll to talk to the C167		
/*		
/*****************************************************************************/
bool Connect(void)
{
	//conntct to driver and register a callback function
	if ( !carapiRecvRegisterCallback(&m_linkId, NULL, MessageHandler) ){
		m_connected = false;
	}
	else{
		m_connected = true;
	}

	return m_connected;
}


/*****************************************************************************/
/*
/*  Funktion:		DisConnect
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:				
/*		
/*****************************************************************************/
void DisConnect(void)
{
	carapiRecvUnregisterCallback(m_linkId);
}


/*****************************************************************************/
/*
/*  Funktion:		SendPCStartUpMessage
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		Message will be sent directly after the service is started
/*					The C167 waits for this message 5 minutes. If this message 
/*					does not arrive the C167 goes into an software error state		
/*		
/*****************************************************************************/
bool SendPCStartUpMessage(void)
{
	DWORD			status		= 0;
	unsigned char	msgBuf[64]	= {0};
	
	DBGTRACE(("SendPCStartUpMessage - entered"));

	if ( !m_connected ){
		DBGTRACE(("SendPCStartUpMessage - failed. Entry condition false."));
		return false;
	}
	
	status = carapiBlockEncf(msgBuf, 
							 sizeof(msgBuf), 
							 "v((vvvv()))", 
							 m_linkId, 
							 POWERMODUL,
							 0, // message handle
							 CARMSG_RegisterClient_CMD,
							 0 // status
							 );

	if( 0 == status ){
		DBGTRACE(("SendPCStartUpMessage - encf failed"));	
		return false;
	}

	status = carapiMsgSend(msgBuf);

	if ( 0 == status ){
		DBGTRACE(("SendPCStartUpMessage - msgSend failed"));	
		return false;
	}
	
	DBGTRACE(("SendPCStartUpMessage - msgSend successful"));
	return true;
}


/*****************************************************************************/
/*
/*  Funktion:		QueryTerm15
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		Queries the current ignition state		
/*		
/*****************************************************************************/
bool QueryTerm15(void)
{
	DWORD			status		= 0;
	unsigned char	msgBuf[64]	= {0};
	
	DBGTRACE(("QueryTerm15 - entered"));

	if ( !m_connected ){
		DBGTRACE(("QueryTerm15 - failed. Entry condition false."));
		return false;
	}

	status = carapiBlockEncf(msgBuf, 
							 sizeof(msgBuf), 
							 "v((vvvv()))", 
							 m_linkId, 
							 POWERMODUL,
							 0, 
							 CARMSG_Ignition_State_CMD,
							 0
							 );

	if( 0 == status ){
		DBGTRACE(("QueryTerm15 - encf failed"));	
		return false;
	}

	status = carapiMsgSend(msgBuf);

	if ( 0 == status ){
		DBGTRACE(("QueryTerm15 - msgSend failed"));	
		return false;
	}
	
	DBGTRACE(("QueryTerm15 - msgSend successful"));
	return true;
}


/*****************************************************************************/
/*
/*  Funktion:		SendSwitchOffMessage
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		This message will switch of the power supply finally
/*					after a given delay		
/*		
/*****************************************************************************/
void SendSwitchOffMessage(void)
{
	DWORD			status		= 0;
	unsigned char	msgBuf[64]	= {0};
	
	DBGTRACE(("SwitchOffPowerSupply - entered"));

	if ( !m_connected ){
		DBGTRACE(("SendSwitchOffMessage - failed. Entry condition false."));
		return;
	}

	status = carapiBlockEncf(msgBuf, 
							 sizeof(msgBuf), 
							 "v((vvvv(v)))", 
							 m_linkId, 
							 POWERMODUL,
							 0, 
							 CARMSG_PowerSupply_SwitchOff_CMD,
							 0, 
							 m_delay // switch off after n milliseconds
							 );

	if( 0 == status ){
		DBGTRACE(("Power_SwitchOffSupply - encf failed"));	
		return;
	}

	status = carapiMsgSend(msgBuf);

	if ( 0 == status ){
		DBGTRACE(("Power_SwitchOffSupply - msgSend failed"));	
		return;
	}
	
	DBGTRACE(("Power_SwitchOffSupply - msgSend successful"));
	return;
}


/*****************************************************************************/
/*
/*  Funktion:		ShutdownSystem
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		force Windows to shut down regardless of currently running
/*					applications, popup-messages or user-privileges.	
/*		
/*****************************************************************************/
void ShutdownSystem(void)
{
	HANDLE hToken = NULL;
	TOKEN_PRIVILEGES tokenPrivileges;

	tokenPrivileges.PrivilegeCount             = 1;
	tokenPrivileges.Privileges[ 0 ].Attributes = (SE_PRIVILEGE_ENABLED);
	
	int success = LookupPrivilegeValue(NULL, 
									   SE_SHUTDOWN_NAME, 
									   &tokenPrivileges.Privileges[0].Luid
									   ); 

	if ( success == false ){
		return;
	}

	success = OpenProcessToken(GetCurrentProcess(), 
							   (TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY), 
							   &hToken 
							   );

	if ( success == false ){
		return;
	}
	
	success = AdjustTokenPrivileges(hToken, 
						  FALSE, 
						  &tokenPrivileges, 
						  0,
						  (PTOKEN_PRIVILEGES) NULL, 
						  0
						  );
	
	if ( success == false ){
		return;
	}
	
	success = ExitWindowsEx(EWX_LOGOFF | EWX_SHUTDOWN | EWX_FORCE, 0);

	if ( success == false ){
		return;
	}
}


/*****************************************************************************/
/*
/*  Funktion:		MessageHandler
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		all messages received by carapi.dll end up here			
/*		
/*****************************************************************************/
void __stdcall MessageHandler(PVOID	a, DWORD b, DWORD c, DWORD d, DWORD	e, PBYTE f)
{
	DWORD value;

	switch ( d ){
	
	//state of ignition changed or an answer to QueryTerm15() was received
	case CARMSG_Ignition_State_EVT:
	case CARMSG_Ignition_State_CON:
		carapiBlockDecf(f, "v",  &value);
		DBGTRACE(("MessageHandler - Ignition State is %d", value));
		
		if ( value == 0 ){
			//we have to quit now, because ignition is off
			DBGTRACE(("MessageHandler - switching off system in %d ms.", m_delay));
			SendSwitchOffMessage();
			DBGTRACE(("MessageHandler - shutting down Windows now."));
			ShutdownSystem();
		}

		break;

	default:
		break;
	}

	return;
}


/*****************************************************************************/
/*
/*  Funktion:		printfToTraceDevice
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		debugging			
/*		
/*****************************************************************************/
void printfToTraceDevice(LPSTR fmt, ...)
{	
	CHAR buf[256];
	va_list args;
	va_start(args, fmt);

	sprintf(buf, "PM: ");
	vsprintf(buf + strlen(buf), fmt, args);
	strcat(buf, "\r\n");
	OutputDebugString(buf);
}
