#include "stdafx.h"
#include "windows.h"
#include "pmService.h"


//*****************************************************************************
//**** global declarations ****************************************************
//*****************************************************************************

TCHAR                   *displayName		= TEXT("Teledrive Powermanagement");
TCHAR                   *serviceName		= TEXT("IAVPM");
SERVICE_STATUS          serviceStatus;
SERVICE_STATUS_HANDLE   serviceStatusHandle = NULL;
HANDLE                  stopServiceEvent	= NULL;
bool					runningAsService	= true;


/*****************************************************************************/
/*
/*  Funktion:		RunService
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		tells the SCM about this service 
/*					(name, function to be called)		
/*		
/*****************************************************************************/
void RunService()
{
	SERVICE_TABLE_ENTRY serviceTable[] =
	{
		{serviceName, ServiceMain},
		{0, 0}
	};

	StartServiceCtrlDispatcher(serviceTable);
}


/*****************************************************************************/
/*
/*  Funktion:		ServiceMain
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		The service control manager (SCM)calls this function to 
/*					start the service		
/*		
/*****************************************************************************/
void WINAPI ServiceMain( DWORD /*argc*/, TCHAR* /*argv*/[] )
{	
	DWORD lastErr;

	DBGTRACE(("ServiceMain entered"));
	
	serviceStatusHandle = RegisterServiceCtrlHandler(serviceName, ServiceControlHandler);

	if ( serviceStatusHandle == NULL ){
		lastErr = GetLastError();
		DBGTRACE(("ServiceMain - could not create serviceStatusHandle"));
		PrintToMessageLog("Registering serviceStatusHandle failed.");
		ReportStatusToSCMgr(SERVICE_STOPPED, lastErr, 0);
		return;
	}
	else{
		serviceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
		serviceStatus.dwServiceSpecificExitCode = 0;
		ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000);
	}

	//Event zum anhalten des Dienstes erzeugen
	stopServiceEvent = CreateEvent(NULL, FALSE, FALSE, NULL);

	if ( stopServiceEvent  == NULL ){
		lastErr = GetLastError();
		DBGTRACE(("ServiceMain - could not create stopServiceEvent"));
		PrintToMessageLog("Registering stopServiceEvent failed.");
		ReportStatusToSCMgr(SERVICE_STOPPED, lastErr, 0);
		return;
	}
	else{
		ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000);
	}

    // Vorbereitungen fr den Dienst
    bool b = MakePreparations();
    
	if ( b == false ){
		ReportStatusToSCMgr(SERVICE_STOPPED, ERROR_BAD_ENVIRONMENT, 0);
		PrintToMessageLog("Service preparations failed. Maybe the driver (carapi_dry.sys) was not started.");
		return;
	}
	
	// running
	ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 0);
	DBGTRACE(("ServiceMain - service successful started"));

	//wait for end
	WaitForSingleObject(stopServiceEvent, INFINITE);

	DBGTRACE(("ServiceMain - Stoping service"));

	// service was stopped
	ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000);

    CleanUp();
    
	// do cleanup here
	CloseHandle( stopServiceEvent );
	stopServiceEvent = NULL;

	// service is now stopped
	serviceStatus.dwControlsAccepted &= ~(SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
	serviceStatus.dwCurrentState = SERVICE_STOPPED;
	SetServiceStatus( serviceStatusHandle, &serviceStatus );
	DBGTRACE(("ServiceMain - service successful stopped"));
	DBGTRACE(("ServiceMain - left"));
}


/*****************************************************************************/
/*
/*  Funktion:		ServiceControlHandler
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		All messages from the service control manager end up here		
/*		
/*****************************************************************************/
void WINAPI ServiceControlHandler(DWORD controlCode)
{
	switch (controlCode)
	{
		case SERVICE_CONTROL_INTERROGATE:
			DBGTRACE(("ServiceControlHandler - SERVICE_CONTROL_INTERROGATE"));
			break;

		case SERVICE_CONTROL_SHUTDOWN:
		case SERVICE_CONTROL_STOP:
			DBGTRACE(("ServiceControlHandler - SERVICE_CONTROL_SHUTDOWN/STOP"));
			ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 0);
			SetEvent(stopServiceEvent);
			return;

		case SERVICE_CONTROL_PAUSE:
			DBGTRACE(("ServiceControlHandler - SERVICE_CONTROL_PAUSE"));
			break;

		case SERVICE_CONTROL_CONTINUE:
			DBGTRACE(("ServiceControlHandler - SERVICE_CONTROL_CONTINUE"));
			break;

		default:
			DBGTRACE(("ServiceControlHandler - unknown control code"));
			break;
	}

	ReportStatusToSCMgr(serviceStatus.dwCurrentState, NO_ERROR, 0);
} 


/*****************************************************************************/
/*
/*  Funktion:		InstallService
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		Installs the service. After next bootup it will be active
/*					if the driver (!) was started		
/*		
/*****************************************************************************/
void InstallService()
{
	SC_HANDLE   serviceControlManager = OpenSCManager(NULL, NULL, SC_MANAGER_CREATE_SERVICE);

	if (serviceControlManager != NULL)
	{
		char path[_MAX_PATH + 1];
		if (GetModuleFileName(NULL, path, sizeof(path)/sizeof(path[0])) > 0)
		{
			SC_HANDLE service = CreateService(
                                    serviceControlManager,
							        serviceName, 
                                    displayName,
							        SERVICE_ALL_ACCESS, 
                                    SERVICE_WIN32_OWN_PROCESS,
							        SERVICE_AUTO_START, 
                                    SERVICE_ERROR_IGNORE, 
                                    path,
							        NULL, 
                                    NULL, 
                                    NULL, 
                                    NULL, 
                                    NULL);
			if (service != NULL)
            {
				CloseServiceHandle(service);
            }
		}

		CloseServiceHandle(serviceControlManager);
	}
}


/*****************************************************************************/
/*
/*  Funktion:		UninstallService
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		The service will be erased from the registry. After next
/*					bootup this service does not start.		
/*		
/*****************************************************************************/
void UninstallService()
{
	SC_HANDLE   serviceControlManager = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);

	if (serviceControlManager != NULL)
	{
		SC_HANDLE service = OpenService(
                                serviceControlManager,
			                    serviceName, 
                                SERVICE_QUERY_STATUS | DELETE);
		if (service != NULL)
		{
			SERVICE_STATUS serviceStatus;
			if (QueryServiceStatus(service, &serviceStatus) == TRUE)
			{
				if (serviceStatus.dwCurrentState == SERVICE_STOPPED)
                {
					DeleteService( service );
                }
			}

			CloseServiceHandle(service);
		}
 
		CloseServiceHandle(serviceControlManager);
	}
}


/*****************************************************************************/
/*
/*  Funktion:		MakePreparations
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		connects to carapi-dll and sends a startup-message. This
/*					tells the C167 that the PC is alive. Then an ignition-query
/*					is sent to see if ignition is still on.			
/*		
/*****************************************************************************/
bool MakePreparations(void)
{
	bool retVal;
	
	retVal = Connect();
	if ( retVal == false ){
		DBGTRACE(("MakePreparations - connect to carapi driver failed"));
		return false;
	}

	retVal = SendPCStartUpMessage();
	if ( retVal == false ){
		DBGTRACE(("MakePreparations - SendPCStartUpMessage failed"));
		return false;
	}

	retVal = QueryTerm15();
	if ( retVal == false ){
		DBGTRACE(("MakePreparations - QueryTerm15 failed"));
		return false;
	}

	return true;
}


/*****************************************************************************/
/*
/*  Funktion:		CleanUp
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:				
/*		
/*****************************************************************************/
void CleanUp(void)
{
	DisConnect();
}


/*****************************************************************************/
/*
/*  Funktion:		ReportStatusToSCMgr
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:				
/*		
/*****************************************************************************/
bool ReportStatusToSCMgr(DWORD dwCurrentState,
                         DWORD dwWin32ExitCode,
                         DWORD dwWaitHint)
{
    static DWORD dwCheckPoint = 1;
    bool fResult = TRUE;

    
    if (dwCurrentState == SERVICE_START_PENDING)
        serviceStatus.dwControlsAccepted = 0;
    else
        serviceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;

    serviceStatus.dwCurrentState = dwCurrentState;
    serviceStatus.dwWin32ExitCode = dwWin32ExitCode;
    serviceStatus.dwWaitHint = dwWaitHint;

    if ( ( dwCurrentState == SERVICE_RUNNING ) ||
         ( dwCurrentState == SERVICE_STOPPED ) )
        serviceStatus.dwCheckPoint = 0;
    else
        serviceStatus.dwCheckPoint = dwCheckPoint++;

    // Report the status of the service to the service control manager.
	SetServiceStatus(serviceStatusHandle, &serviceStatus);
    
    return fResult;
}


/*****************************************************************************/
/*
/*  Funktion:		PrintToMessageLog
/*	
/*
/*	Parameters:		--
/*
/*	Return Value:	--
/*
/*	Comments:		adds a trace to the windows event log		
/*		
/*****************************************************************************/
void PrintToMessageLog(LPTSTR lpszMsg)
{	
	DWORD   dwErr;
    TCHAR   szMsg[256];
    HANDLE  hEventSource;
    LPCTSTR  lpszStrings[2];
   
    dwErr = GetLastError();

    hEventSource = RegisterEventSource(NULL, serviceName);

    sprintf(szMsg, TEXT("%s error: %d"), serviceName, dwErr);
    lpszStrings[0] = szMsg;
    lpszStrings[1] = lpszMsg;

    if (hEventSource != NULL) {
        ReportEvent(hEventSource, // handle of event source
            EVENTLOG_ERROR_TYPE,  // event type
            0,                    // event category
            0,                    // event ID
            NULL,                 // current user's SID
            2,                    // strings in lpszStrings
            0,                    // no bytes of raw data
            lpszStrings,          // array of error strings
            NULL);                // no raw data

        DeregisterEventSource(hEventSource);
    }
}